@@ -1,4 +1,4 @@
-// $Id: debugger.h,v 1.41 1999/12/18 13:39:12 jsixt Exp $
+// $Id: debugger.h,v 1.66 2000/08/18 19:22:48 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -6,12 +6,8 @@
 #ifndef DEBUGGER_H
 #define DEBUGGER_H
 
-#include <qqueue.h>
 #include <qtimer.h>
-#include <qfile.h>
 #include <qdict.h>
-#include <kprocess.h>
-#include "brkpt.h"
 #include "envvar.h"
 #include "exprwnd.h"			/* some compilers require this */
 
@@ -24,17 +20,19 @@ class VarTree;
 class ProgramTypeTable;
 class KTreeViewItem;
 class KConfig;
+class KSimpleConfig;
 class QListBox;
+class RegisterInfo;
+class ThreadInfo;
+class DebuggerDriver;
+class CmdQueueItem;
+class Breakpoint;
+struct DisassembledCode;
+struct MemoryDump;
+struct DbgAddr;
+class KProcess;
 
 
-class GdbProcess : public KProcess
-{
-public:
-    GdbProcess();
-protected:
-    virtual int commSetupDoneC();
-};
-
 class KDebugger : public QObject
 {
     Q_OBJECT
@@ -42,28 +40,25 @@ public:
     KDebugger(QWidget* parent,		/* will be used as the parent for dialogs */
 	      ExprWnd* localVars,
 	      ExprWnd* watchVars,
-	      QListBox* backtrace
+	      QListBox* backtrace,
+	      DebuggerDriver* driver
 	      );
     ~KDebugger();
     
     /**
-     * This function starts to debug the specified executable.
-     * @return false if an error occurs, in particular if a program is
-     * currently being debugged
+     * This function starts to debug the specified executable. If a program
+     * is currently being debugged, it is terminated first.
+     *
+     * @return false if an error occurs.
      */
     bool debugProgram(const QString& executable);
 
     /**
-     * Queries the user for an executable file and debugs it. If a program
-     * is currently being debugged, it is terminated first.
+     * Uses the specified core to debug the active program.
+     * @param batch tells whether the core file was given on the
+     * command line.
      */
-    void fileExecutable();
-
-    /**
-     * Queries the user for a core file and uses it to debug the active
-     * program
-     */
-    void fileCoreFile();
+    void useCoreFile(QString corefile, bool batch);
 
     /**
      * Runs the program or continues it if it is stopped at a breakpoint.
@@ -71,9 +66,9 @@ public:
     void programRun();
 
     /**
-     * Attaches to a process and debugs it.
+     * Attaches to the specified process and debugs it.
      */
-    void programAttach();
+    void attachProgram(const QString& pid);
 
     /**
      * Restarts the debuggee.
@@ -82,19 +77,26 @@ public:
 
     /**
      * Performs a single-step, possibly stepping into a function call.
+     * If byInsn is true, a step by instruction is performed.
      */
-    void programStep();
+    void programStep(bool byInsn);
 
     /**
      * Performs a single-step, stepping over a function call.
+     * If byInsn is true, a step by instruction is performed.
      */
-    void programNext();
+    void programNext(bool byInsn);
 
     /**
      * Runs the program until it returns from the current function.
      */
     void programFinish();
 
+    /**
+     * Kills the program (removes it from memory).
+     */
+    void programKill();
+
     /**
      * Interrupts the program if it is currently running.
      */
@@ -103,19 +105,19 @@ public:
     /**
      * Queries the user for program arguments.
      */
-    void programArgs();
+    void programArgs(QWidget* parent);
+
+    /**
+     * Queries the user for program settings: Debugger command, terminal
+     * emulator.
+     */
+    void programSettings(QWidget* parent);
 
     /**
      * Setup remote debugging device
      */
     void setRemoteDevice(const QString& remoteDevice) { m_remoteDevice = remoteDevice; }
 
-    /**
-     * Shows the breakpoint list if it isn't currently visible or hides it
-     * if it is.
-     */
-    void breakListToggleVisible();
-
     /**
      * Run the debuggee until the specified line in the specified file is
      * reached.
@@ -127,19 +129,28 @@ public:
 
     /**
      * Set a breakpoint.
-     * 
+     *
+     * @param fileName The source file in which to set the breakpoint.
+     * @param lineNo The zero-based line number.
+     * @param address The exact address of the breakpoint.
+     * @param temporary Specifies whether this is a temporary breakpoint
      * @return false if the command was not executed, e.g. because the
      * debuggee is running at the moment.
      */
-    bool setBreakpoint(const QString& fileName, int lineNo, bool temporary);
+    bool setBreakpoint(QString fileName, int lineNo,
+		       const DbgAddr& address, bool temporary);
 
     /**
      * Enable or disable a breakpoint at the specified location.
      * 
+     * @param fileName The source file in which the breakpoint is.
+     * @param lineNo The zero-based line number.
+     * @param address The exact address of the breakpoint.
      * @return false if the command was not executed, e.g. because the
      * debuggee is running at the moment.
      */
-    bool enableDisableBreakpoint(const QString& fileName, int lineNo);
+    bool enableDisableBreakpoint(QString fileName, int lineNo,
+				 const DbgAddr& address);
 
     /**
      * Tells whether one of the single stepping commands can be invoked
@@ -155,7 +166,7 @@ public:
     /**
      * Tells whether the debuggee can be changed.
      */
-    bool canChangeExecutable() { return isReady() && !m_programActive; }
+    bool canUseCoreFile() { return isReady() && !m_programActive; }
 
     /**
      * Add a watch expression.
@@ -166,12 +177,12 @@ public:
      * Retrieves the current status message.
      */
     const QString& statusMessage() const { return m_statusMessage; }
+
     /**
      * Is the debugger ready to receive another high-priority command?
      */
-    bool isReady() const { return m_haveExecutable &&
-	    /*(m_state == DSidle || m_state == DSrunningLow)*/
-	    m_hipriCmdQueue.isEmpty(); }
+    bool isReady() const;
+
     /**
      * Is the debuggee running (not just active)?
      */
@@ -187,181 +198,98 @@ public:
      */
     bool isProgramActive() { return m_programActive; }
 
-    /** Is the breakpoint table visible? */
-    bool isBreakListVisible() { return m_bpTable.isVisible(); }
+    /**
+     * Is the debugger driver idle?
+     */
+    bool isIdle() const;
 
-    /** The table of breakpoints. */
-    BreakpointTable& breakList() { return m_bpTable; }
+    /** The list of breakpoints. */
+    int numBreakpoints() const { return m_brkpts.size(); }
+    const Breakpoint* breakpoint(int i) const { return m_brkpts[i]; }
 
     const QString& executable() const { return m_executable; }
 
-    void setCoreFile(const QString& corefile) { m_corefile = corefile; }
-
     /**
      * Sets the command to invoke gdb. If cmd is the empty string, the
      * default is substituted.
      */
-    void setDebuggerCmd(const QString& cmd);
-
-    /** Retrieves the command to invoke gdb. */
-    QString debuggerCmd() const { return m_debuggerCmdStr; }
+    void setDebuggerCmd(const QString& cmd)
+    { m_generalDebuggerCmd = cmd; }
 
     /**
-     * Sets the command to invoke the terminal that displays the program
-     * output. If cmd is the empty string, the default is substituted.
+     * Terminal emulation level.
      */
-    void setTerminalCmd(const QString& cmd);
+    enum TTYLevel {
+	ttyNone = 0,			/* ignore output, input triggers EOF */
+	ttySimpleOutputOnly = 1,	/* minmal output emulation, input triggers EOF */
+	ttyFull = 7			/* program needs full emulation */
+    };
 
     /**
-     * Retrieves the command to invoke ther terminal that displays the
-     * program output.
+     * Returns the level of terminal emulation requested by the inferior.
      */
-    QString terminalCmd() const { return m_outputTermCmdStr; }
+    TTYLevel ttyLevel() const { return m_ttyLevel; }
+
+    /** Sets the terminal that is to be used by the debugger. */
+    void setTerminal(const QString& term) { m_inferiorTerminal = term; }
+
+    /** Returns the debugger driver. */
+    DebuggerDriver* driver() { return m_d; }
+
+    /** Returns the pid that the debugger is currently attached to. */
+    const QString& attachedPid() const { return m_attachedPid; }
+
+    /**
+     * The memory at that the expression evaluates to is watched. Can be
+     * empty. Triggers a redisplay even if the expression did not change.
+     */
+    void setMemoryExpression(const QString& memexpr);
+
+    /**
+     * Sets how the watched memory location is displayed.
+     * Call setMemoryExpression() to force a redisplay.
+     */
+    void setMemoryFormat(unsigned format) { m_memoryFormat = format; }
 
     // settings
     void saveSettings(KConfig*);
     void restoreSettings(KConfig*);
 
-public:
-    // debugger driver
-    enum DbgCommand {
-	DCinitialize,
-	DCinitialSet,
-	DCexecutable,
-	DCcorefile,
-	DCattach,
-	DCinfolinemain,
-	DCinfolocals,
-	DCinfoargs,
-	DCsetargs,
-	DCsetenv,
-	DCcd,
-	DCbt,
-	DCrun,
-	DCcont,
-	DCstep,
-	DCnext,
-	DCfinish,
-	DCuntil,
-	DCbreak,
-	DCdelete,
-	DCenable,
-	DCdisable,
-	DCprint,
-	DCprintStruct,
-	DCprintQStringStruct,
-	DCframe,
-	DCfindType,
-	DCinfosharedlib,
-	DCinfobreak,
-	DCcondition,
-	DCignore
-    };
 protected:
-    QString m_outputTermCmdStr;
-    pid_t m_outputTermPID;
-    QString m_outputTermName;
-    QString m_outputTermKeepScript;
-    bool createOutputWindow();
-    QString m_debuggerCmdStr;
-    bool startGdb();
-    void stopGdb();
+    QString m_inferiorTerminal;
+    QString m_debuggerCmd;		/* per-program setting */
+    QString m_generalDebuggerCmd;	/* global setting */
+    TTYLevel m_ttyLevel;		/* level of terminal emulation */
+    bool startDriver();
+    void stopDriver();
     void writeCommand();
     
-    enum DebuggerState {
-	DSidle,				/* gdb waits for input */
-	DSinterrupted,			/* a command was interrupted */
-	DSrunningLow,			/* gdb is running a low-priority command */
-	DSrunning,			/* gdb waits for program */
-	DScommandSent,			/* command has been sent, we wait for wroteStdin signal */
-	DScommandSentLow		/* low-prioritycommand has been sent */
-    };
-    DebuggerState m_state;
-    char* m_gdbOutput;			/* normal gdb output */
-    int m_gdbOutputLen;			/* current accumulated output */
-    int m_gdbOutputAlloc;		/* space available in m_gdbOutput */
-#if QT_VERSION < 200
-    typedef QString DelayedStr;
-#else
-    typedef QCString DelayedStr;
-#endif
-    QQueue<DelayedStr> m_delayedOutput;	/* output colleced while we have receivedOutput */
-					/* but before signal wroteStdin arrived */
     QList<VarTree> m_watchEvalExpr;	/* exprs to evaluate for watch windows */
-    QList<VarTree> m_parsedLocals;	/* local variables that have just been parsed */
-
-public:
-    /**
-     * Gdb commands are placed in a queue. Only one command at a time is
-     * sent down to gdb. All other commands in the queue are retained until
-     * the sent command has been processed by gdb. Gdb tells us that it's
-     * done with the command by sending the prompt. The output of gdb is
-     * parsed at that time. Then, if more commands are in the queue, the
-     * next one is sent to gdb.
-     * 
-     * The active command is kept separately from other pending commands.
-     */
-    struct CmdQueueItem
-    {
-	DbgCommand m_cmd;
-	QString m_cmdString;
-	bool m_committed;		/* just a debugging aid */
-	// remember which expression when printing an expression
-	VarTree* m_expr;
-	ExprWnd* m_exprWnd;
-	// whether command was emitted due to direct user request (only set when relevant)
-	bool m_byUser;
-
-	CmdQueueItem(DbgCommand cmd, const QString& str) :
-		m_cmd(cmd),
-		m_cmdString(str),
-		m_committed(false),
-		m_expr(0),
-		m_exprWnd(0),
-		m_byUser(false)
-	{ }
-    };
-    /**
-     * Enqueues a high-priority command. High-priority commands are
-     * executed before any low-priority commands. No user interaction is
-     * possible as long as there is a high-priority command in the queue.
-     */
-    CmdQueueItem* executeCmd(DbgCommand cmd, QString cmdString, bool clearLow = false);
-    enum QueueMode {
-	QMnormal,			/* queues the command last */
-	QMoverride,			/* removes an already queued command */
-	QMoverrideMoreEqual		/* ditto, also puts the command first in the queue */
-    };
-    /**
-     * Enqueues a low-priority command. Low-priority commands are executed
-     * after any high-priority commands.
-     */
-    CmdQueueItem* queueCmd(DbgCommand cmd, QString cmdString, QueueMode mode);
-    /** Removes all commands from the low-priority queue. */
-    void flushLoPriQueue();
-    /** Removes all commands from  the high-priority queue. */
-    void flushHiPriQueue();
+    QArray<Breakpoint*> m_brkpts;
+    QString m_memoryExpression;		/* memory location to watch */
+    unsigned m_memoryFormat;		/* how that output should look */
 
+protected slots:
+    void parse(CmdQueueItem* cmd, const char* output);
 protected:
-    QQueue<CmdQueueItem> m_hipriCmdQueue;
-    QList<CmdQueueItem> m_lopriCmdQueue;
-    CmdQueueItem* m_activeCmd;		/* the cmd we are working on */
-    void parse(CmdQueueItem* cmd);
-    VarTree* parseExpr(const char* name, bool wantErrorValue = true);
-    VarTree* parseQCharArray(const char* name, bool wantErrorValue);
-    void handleRunCommands();
+    VarTree* parseExpr(const char* output, bool wantErrorValue);
+    void handleRunCommands(const char* output);
     void updateAllExprs();
-    void updateBreakptTable();
     void updateProgEnvironment(const QString& args, const QString& wd,
 			       const QDict<EnvVar>& newVars);
-    void parseLocals(QList<VarTree>& newVars);
-    void handleLocals();
-    bool handlePrint(CmdQueueItem* cmd);
-    void handleBacktrace();
-    void handleFrameChange();
-    void handleFindType(CmdQueueItem* cmd);
-    void handlePrintStruct(CmdQueueItem* cmd);
-    void handleSharedLibs();
+    void parseLocals(const char* output, QList<VarTree>& newVars);
+    void handleLocals(const char* output);
+    bool handlePrint(CmdQueueItem* cmd, const char* output);
+    void handleBacktrace(const char* output);
+    void handleFrameChange(const char* output);
+    void handleFindType(CmdQueueItem* cmd, const char* output);
+    void handlePrintStruct(CmdQueueItem* cmd, const char* output);
+    void handleSharedLibs(const char* output);
+    void handleRegisters(const char* output);
+    void handleMemoryDump(const char* output);
+    void handleInfoLine(CmdQueueItem* cmd, const char* output);
+    void handleDisassemble(CmdQueueItem* cmd, const char* output);
+    void handleThreadList(const char* output);
     void evalExpressions();
     void evalInitialStructExpression(VarTree* var, ExprWnd* wnd, bool immediate);
     void evalStructExpression(VarTree* var, ExprWnd* wnd, bool immediate);
@@ -369,9 +297,18 @@ protected:
     void dereferencePointer(ExprWnd* wnd, VarTree* var, bool immediate);
     void determineType(ExprWnd* wnd, VarTree* var);
     void removeExpr(ExprWnd* wnd, VarTree* var);
+    void queueMemoryDump(bool immediate);
     CmdQueueItem* loadCoreFile();
+    void openProgramConfig(const QString& name);
+
+    Breakpoint* breakpointByFilePos(QString file, int lineNo,
+				    const DbgAddr& address);
+    void newBreakpoint(const char* output);
+    void updateBreakList(const char* output);
+    bool stopMayChangeBreakList() const;
+    void saveBreakpoints(KSimpleConfig* config);
+    void restoreBreakpoints(KSimpleConfig* config);
 
-    QString m_lastDirectory;		/* the dir of the most recently opened file */
     bool m_haveExecutable;		/* has an executable been specified */
     bool m_programActive;		/* is the program active (possibly halting in a brkpt)? */
     bool m_programRunning;		/* is the program executing (not stopped)? */
@@ -385,36 +322,38 @@ protected:
     QDict<EnvVar> m_envVars;		/* environment variables set by user */
     QStrList m_sharedLibs;		/* shared libraries used by program */
     ProgramTypeTable* m_typeTable;	/* known types used by the program */
-    bool m_qstring2nullOk;		/* whether gdb knows about QString::null */
     KSimpleConfig* m_programConfig;	/* program-specific settings (brkpts etc) */
     void saveProgramSettings();
     void restoreProgramSettings();
 
     // debugger process
-    GdbProcess m_gdb;
-    int m_gdbMajor, m_gdbMinor;
+    DebuggerDriver* m_d;
     bool m_explicitKill;		/* whether we are killing gdb ourselves */
 
-#ifdef GDB_TRANSCRIPT
-    // log file
-    QFile m_logFile;
-#endif
-
     QString m_statusMessage;
 
 protected slots:
-    void receiveOutput(KProcess*, char* buffer, int buflen);
-    void commandRead(KProcess*);
     void gdbExited(KProcess*);
+    void slotInferiorRunning();
+    void backgroundUpdate();
     void gotoFrame(int);
     void slotLocalsExpanding(KTreeViewItem*, bool&);
     void slotWatchExpanding(KTreeViewItem*, bool&);
-    void slotToggleBreak(const QString&, int);
-    void slotEnaDisBreak(const QString&, int);
     void slotUpdateAnimation();
     void slotDeleteWatch();
-    
+    void slotValuePopup(const QString&);
+    void slotDisassemble(const QString&, int);
+public slots:
+    void setThread(int);
+    void shutdown();
+
 signals:
+    /**
+     * This signal is emitted before the debugger is started. The slot is
+     * supposed to set up m_inferiorTerminal.
+     */
+    void debuggerStarting();
+
     /**
      * This signal is emitted whenever a part of the debugger needs to
      * highlight the specfied source code line (e.g. when the program
@@ -426,8 +365,9 @@ signals:
      * @param lineNo specifies the line number (0-based!) (this may be
      * negative, in which case the file should be activated, but the line
      * should NOT be changed).
+     * @param address specifies the exact address of the PC or is empty.
      */
-    void activateFileLine(const QString& file, int lineNo);
+    void activateFileLine(const QString& file, int lineNo, const DbgAddr& address);
 
     /**
      * This signal is emitted when a line decoration item (those thingies
@@ -442,12 +382,15 @@ signals:
      * @param filename specifies the filename where the program stopped
      * @param lineNo specifies the line number (zero-based); it can be -1
      * if it is unknown
+     * @param address specifies the address that the instruction pointer
+     * points to.
      * @param frameNo specifies the frame number: 0 is the innermost frame,
      * positive numbers are frames somewhere up the stack (indicates points
      * where a function was called); the latter cases should be indicated
      * differently in the source window.
      */
-    void updatePC(const QString& filename, int lineNo, int frameNo);
+    void updatePC(const QString& filename, int lineNo,
+		  const DbgAddr& address, int frameNo);
 
     /**
      * This signal is emitted when gdb detects that the executable has been
@@ -473,21 +416,67 @@ signals:
      */
     void updateUI();
 
+    /**
+     * Indicates that the list of breakpoints has possibly changed.
+     */
+    void breakpointsChanged();
+
+    /**
+     * Indicates that the register values have possibly changed.
+     */
+    void registersChanged(QList<RegisterInfo>&);
+
+    /**
+     * Indicates that the list of threads has possibly changed.
+     */
+    void threadsChanged(QList<ThreadInfo>&);
+
+    /**
+     * Indicates that the value for a value popup is ready.
+     */
+    void valuePopup(const QString&);
+
+    /**
+     * Provides the disassembled code of the location given by file and
+     * line number (zero-based).
+     */
+    void disassembled(const QString& file, int line, const QList<DisassembledCode>& code);
+
+    /**
+     * Indicates that the program has stopped for any reason: by a
+     * breakpoint, by a signal that the debugger driver caught, by a single
+     * step instruction.
+     */
+    void programStopped();
+
+    /**
+     * Indicates that a new memory dump output is ready.
+     * @param msg is an error message or empty
+     * @param memdump is the memory dump
+     */
+    void memoryDumpChanged(const QString&, QList<MemoryDump>&);
+
+    /**
+     * Gives other objects a chance to save program specific settings.
+     */
+    void saveProgramSpecific(KSimpleConfig* config);
+
+    /**
+     * Gives other objects a chance to restore program specific settings.
+     */
+    void restoreProgramSpecific(KSimpleConfig* config);
+
 protected:
-    BreakpointTable m_bpTable;
     ExprWnd& m_localVariables;
     ExprWnd& m_watchVariables;
     QListBox& m_btWindow;
-    
+
     // animation
     QTimer m_animationTimer;
     int m_animationInterval;
     
     // implementation helpers
 protected:
-    void initMenu();
-    void initToolbar();
-    void initAnimation();
     void startAnimation(bool fast);
     void stopAnimation();
 
