@@ -1,11 +1,10 @@
-// $Id: exprwnd.cpp,v 1.17 1999/11/14 16:59:06 jsixt Exp $
+// $Id: exprwnd.cpp,v 1.19 2000/01/06 16:46:15 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
 
 #include "exprwnd.h"
 #include "exprwnd.moc"
-#include "parsevar.h"
 #include "typetable.h"
 #include <qstrlist.h>
 #include <qpainter.h>
@@ -146,6 +145,9 @@ void VarTree::inferTypesOfChildren(ProgramTypeTable& typeTable)
 
     // if this is a pointer, get the type from the value (less the pointer)
     if (m_varKind == VKpointer) {
+#ifndef I_know_a_way_to_do_this_cleanly
+	return;
+#else
 	const char* p = m_value.data();
 	const char* start = p;
 	// the type of the pointer shows up in the value (sometimes)
@@ -162,16 +164,13 @@ void VarTree::inferTypesOfChildren(ProgramTypeTable& typeTable)
 	    return;
 	}
 	const QString& typeName =
-#if QT_VERSION < 200
-	    QString(start+1, p-start-3+1) // minus 3 chars plus '\0'
-#else
-	    QString::fromLatin1(start+1, p-start-3) // minus 3 chars
-#endif
+	    FROM_LATIN1(start+1, p-start-3) // minus 3 chars
 		.stripWhiteSpace();
 	m_type = typeTable.lookup(typeName);
 	if (m_type == 0) {
 	    m_type = TypeInfo::unknownType();
 	}
+#endif
     } else if (m_varKind == VKstruct) {
 	// check if this is a base class part
 	if (m_nameKind == NKtype) {
