@@ -1,5 +1,5 @@
 /*
- * $Id: ktreeview.cpp,v 1.10 1999/12/11 18:17:46 jsixt Exp $
+ * $Id: ktreeview.cpp,v 1.12 2000/11/11 11:12:29 jsixt Exp $
  *
  * KTreeView implementation
  * 
@@ -602,6 +602,7 @@ KTreeView::KTreeView(QWidget *parent,
 	goingDown(false),
 	itemIndent(18),
 	showText(true),
+	moveCurrentToSibling(false),
 	itemCapacity(500),
 	visibleItems(0),
 	rubberband_mode(false)
@@ -623,7 +624,11 @@ KTreeView::KTreeView(QWidget *parent,
 	setFrameStyle(QFrame::Panel | QFrame::Plain);
 	setLineWidth(1);
     }
+#if QT_VESION < 200
     setFocusPolicy(StrongFocus);
+#else
+    setFocusPolicy(WheelFocus);
+#endif
     treeRoot = new KTreeViewItem;
     treeRoot->setExpanded(true);
     treeRoot->owner = this;
@@ -1205,6 +1210,11 @@ void KTreeView::setIndentSpacing(int spacing)
 	repaint();
 }
 
+void KTreeView::setMoveCurrentToSibling(bool m)
+{
+    moveCurrentToSibling = m;
+}
+
 // enables/disables vertical scrollbar
 void KTreeView::setScrollBar(bool enable)
 {
@@ -2039,6 +2049,7 @@ void KTreeView::takeItem(KTreeViewItem* item)
      * the taken-out subtree is at least partially visible.
      */
     KTreeViewItem* cur = current >= 0  ?  itemAt(current)  :  0;
+    KTreeViewItem* oldCurrent = cur;
     if (wasVisible && cur != 0) {
 	KTreeViewItem* c = cur;
 	while (c != 0 && c != item) {
@@ -2047,8 +2058,19 @@ void KTreeView::takeItem(KTreeViewItem* item)
 	if (c != 0) {
 	    // move current item to parent
 	    cur = item->getParent();
-	    if (cur == treeRoot)
+	    if (cur == treeRoot) {
 		cur = 0;
+		// move it to next or previous sibling
+		if (moveCurrentToSibling) {
+		    if (item->getSibling() != 0) {
+			cur = item->getSibling();
+		    } else if (treeRoot->getChild() != item) {
+			cur = treeRoot->getChild();
+			while (cur != 0 && cur->getSibling() != item)
+			    cur = cur->getSibling();
+		    }
+		}
+	    }
 	}
     }
     KTreeViewItem* parentItem = item->getParent();
@@ -2064,7 +2086,13 @@ void KTreeView::takeItem(KTreeViewItem* item)
     }
 
     // re-seat the current item
-    setCurrentItem(cur != 0  ?  itemRow(cur)  :  -1);
+    // row changes if cur is below the taken item
+    current = cur != 0  ?  itemRow(cur)  :  -1;
+    // signal must be emitted only if item really changed
+    if (cur != oldCurrent) {
+	updateCell(current, 0, false);
+	emit highlighted(current);
+    }
 }
 
 // visits each item, calculates the maximum width  
