@@ -1,5 +1,5 @@
 /*
- * $Id: ktreeview.h,v 1.9 1999/12/11 18:17:46 jsixt Exp $
+ * $Id: ktreeview.h,v 1.10 2000/06/18 15:26:39 jsixt Exp $
  * 
  * KTreeView class interface
  * 
@@ -826,6 +826,14 @@ public:
      */
     void setIndentSpacing(int spacing);
 
+    /**
+     * If true, removing a top-level item that contains the current item
+     * will move the current item to the following sibling (or to the
+     * previous if there is none). Otherwise, there will not be a current
+     * item.
+     */
+    void setMoveCurrentToSibling(bool m = true);
+
   /**
 	If enable is TRUE, displays a vertical scroll bar, else hides it.                                        
 	*/
@@ -1124,6 +1132,7 @@ protected:
     int itemIndent;
     int maxItemWidth;
     bool showText;
+    bool moveCurrentToSibling;
     // list of visible items
     int itemCapacity;			/* for how many items we've space allocated */
     KTreeViewItem** visibleItems;
