@@ -1,4 +1,4 @@
-// $Id: pgmargs.cpp,v 1.13 1999/12/12 20:59:07 jsixt Exp $
+// $Id: pgmargs.cpp,v 1.14 2000/11/11 11:12:29 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -73,8 +73,6 @@ PgmArgs::PgmArgs(QWidget* parent, const QString& pgm, QDict<EnvVar>& envVars) :
     m_envList.setMinimumSize(330, 40);
     m_envList.addColumn(i18n("Name"), 100);
     m_envList.addColumn(i18n("Value"), 260);
-    /* work around a non-feature of KTabListBox: */
-    m_envList.setFocusPolicy(QWidget::StrongFocus);
     connect(&m_envList, SIGNAL(currentChanged(QListViewItem*)),
 	    SLOT(envListCurrentChanged(QListViewItem*)));
     initEnvList();
