@@ -1,4 +1,4 @@
-// $Id: textvw.h,v 1.8 1999/12/11 18:17:46 jsixt Exp $
+// $Id: textvw.h,v 1.9 2000/05/20 15:19:42 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -26,10 +26,14 @@ protected:
     virtual void activateLine(int row);
     virtual int textCol() const;
     virtual bool updateCellSize(const QString& text);
+    virtual void setupPainter(QPainter* p);
 
 signals:
     void lineChanged();
 
+public slots:
+    void setTabWidth(int numChars);
+
     // event handling
 protected:
     virtual void keyPressEvent(QKeyEvent* ev);
@@ -40,8 +44,9 @@ protected:
     void paletteChange(const QPalette& oldPal);
 
     int m_width;
-    int m_height;
-    
+    int m_height;			/* line height */
+    int m_tabWidth;			/* in pixels */
+
     ValArray<QString> m_texts;
     int m_curRow;			/* cursor position */
 };
