@@ -1,24 +1,19 @@
-// $Id: winstack.cpp,v 1.28 1999/12/11 18:17:46 jsixt Exp $
+// $Id: winstack.cpp,v 1.41 2000/11/11 11:12:29 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
 
 #include "winstack.h"
-#include "winstack.moc"
 #include "commandids.h"
-#include "brkpt.h"
-#include <kfiledialog.h>
-#include <qtextstream.h>
-#include <qpainter.h>
+#include "sourcewnd.h"
 #include <qbrush.h>
-#include <qfile.h>
 #include <qfileinfo.h>
 #include <qlistbox.h>
 #include <kapp.h>
-#include <kiconloader.h>
-#include <kstdaccel.h>
 #if QT_VERSION >= 200
 #include <klocale.h>			/* i18n */
+#else
+#include <ctype.h>
 #endif
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,350 +21,16 @@
 #include "mydebug.h"
 
 
-// helper that checks for matching file names
-static bool fileNamesMatch(const QString& f1, const QString& f2);
-
-
-FileWindow::FileWindow(const char* fileName, QWidget* parent, const char* name) :
-	KTextView(parent, name),
-	m_fileName(fileName)
-{
-    setNumCols(2);
-    setFont(QFont("courier"));
-
-    // load pixmaps
-#if QT_VERSION < 200
-    KIconLoader* loader = kapp->getIconLoader();
-    m_pcinner = loader->loadIcon("pcinner.xpm");
-    m_pcup = loader->loadIcon("pcup.xpm");
-    m_brkena = loader->loadIcon("brkena.xpm");
-    m_brkdis = loader->loadIcon("brkdis.xpm");
-    m_brktmp = loader->loadIcon("brktmp.xpm");
-    m_brkcond = loader->loadIcon("brkcond.xpm");
-#else
-    m_pcinner = BarIcon("pcinner");
-    m_pcup = BarIcon("pcup");
-    m_brkena = BarIcon("brkena");
-    m_brkdis = BarIcon("brkdis");
-    m_brktmp = BarIcon("brktmp");
-    m_brkcond = BarIcon("brkcond");
-#endif
-}
-
-FileWindow::~FileWindow()
-{
-}
-
-void FileWindow::loadFile()
-{
-    QFile f(m_fileName);
-    if (f.open(IO_ReadOnly)) {
-	QTextStream t(&f);
-	QString s;
-	while (!t.eof()) {
-	    s = t.readLine();
-	    insertLine(s);
-	}
-	f.close();
-    }
-    
-    // allocate line items
-    m_lineItems.fill(0, numRows());
-}
-
-void FileWindow::reloadFile()
-{
-    QFile f(m_fileName);
-    if (!f.open(IO_ReadOnly)) {
-	// open failed; leave alone
-	return;
-    }
-
-    bool autoU = autoUpdate();
-    setAutoUpdate(false);
-
-    QTextStream t(&f);
-    QString s;
-    int lineNo = 0;
-    while (lineNo < m_texts.size() && !t.eof()) {
-	s = t.readLine();
-	replaceLine(lineNo, s);
-	lineNo++;
-    }
-    if (lineNo >= m_texts.size()) {
-	// the new file has more lines than the old one
-	// here lineNo is the number of lines of the old file
-	while (!t.eof()) {
-	    s = t.readLine();
-	    insertLine(s);
-	}
-	// allocate line items
-	m_lineItems.resize(m_texts.size());
-	for (int i = m_texts.size()-1; i >= lineNo; i--) {
-	    m_lineItems[i] = 0;
-	}
-    } else {
-	// the new file has fewer lines
-	// here lineNo is the number of lines of the new file
-	// remove the excessive lines
-	m_texts.setSize(lineNo);
-	m_lineItems.resize(lineNo);
-	// if the cursor is in the deleted lines, move it to the last line
-	if (m_curRow >= lineNo) {
-	    m_curRow = -1;		/* at this point don't have an active row */
-	    activateLine(lineNo-1);	/* now we have */
-	}
-    }
-    f.close();
-
-    setNumRows(m_texts.size());
-
-    setAutoUpdate(autoU);
-    if (autoU && isVisible()) {
-	repaint();
-    }
-}
-
-void FileWindow::scrollTo(int lineNo)
-{
-    if (lineNo >= numRows())
-	lineNo = numRows();
-
-    // scroll to lineNo
-    if (lineNo >= topCell() && lineNo <= lastRowVisible()) {
-	// line is already visible
-	setCursorPosition(lineNo, 0);
-	return;
-    }
-
-    // setCursorPosition does only a half-hearted job of making the
-    // cursor line visible, so we do it by hand...
-    setCursorPosition(lineNo, 0);
-    lineNo -= 5;
-    if (lineNo < 0)
-	lineNo = 0;
-
-    setTopCell(lineNo);
-}
-
-int FileWindow::textCol() const
-{
-    // text is in column 1
-    return 1;
-}
-
-int FileWindow::cellWidth(int col)
-{
-    if (col == 0) {
-	return 15;
-    } else {
-	return KTextView::cellWidth(col);
-    }
-}
-
-void FileWindow::paintCell(QPainter* p, int row, int col)
-{
-    if (col == textCol()) {
-	KTextView::paintCell(p, row, col);
-	return;
-    }
-    if (col == 0) {
-	uchar item = m_lineItems[row];
-	if (item == 0)			/* shortcut out */
-	    return;
-	int h = cellHeight(row);
-	if (item & liBP) {
-	    // enabled breakpoint
-	    int y = (h - m_brkena.height())/2;
-	    if (y < 0) y = 0;
-	    p->drawPixmap(0,y,m_brkena);
-	}
-	if (item & liBPdisabled) {
-	    // disabled breakpoint
-	    int y = (h - m_brkdis.height())/2;
-	    if (y < 0) y = 0;
-	    p->drawPixmap(0,y,m_brkdis);
-	}
-	if (item & liBPtemporary) {
-	    // temporary breakpoint marker
-	    int y = (h - m_brktmp.height())/2;
-	    if (y < 0) y = 0;
-	    p->drawPixmap(0,y,m_brktmp);
-	}
-	if (item & liBPconditional) {
-	    // conditional breakpoint marker
-	    int y = (h - m_brkcond.height())/2;
-	    if (y < 0) y = 0;
-	    p->drawPixmap(0,y,m_brkcond);
-	}
-	if (item & liPC) {
-	    // program counter in innermost frame
-	    int y = (h - m_pcinner.height())/2;
-	    if (y < 0) y = 0;
-	    p->drawPixmap(0,y,m_pcinner);
-	}
-	if (item & liPCup) {
-	    // program counter somewhere up the stack
-	    int y = (h - m_pcup.height())/2;
-	    if (y < 0) y = 0;
-	    p->drawPixmap(0,y,m_pcup);
-	}
-	return;
-    }
-}
-
-void FileWindow::updateLineItems(const BreakpointTable& bpt)
-{
-    // clear outdated breakpoints
-    for (int i = m_lineItems.size()-1; i >= 0; i--) {
-	if (m_lineItems[i] & liBPany) {
-	    // check if this breakpoint still exists
-	    TRACE(QString().sprintf("checking for bp at %d", i));
-	    int j;
-	    for (j = bpt.numBreakpoints()-1; j >= 0; j--) {
-		const Breakpoint& bp = bpt.breakpoint(j);
-		if (bp.lineNo == i && fileNamesMatch(bp.fileName, fileName())) {
-		    // yes it exists; mode is changed below
-		    break;
-		}
-	    }
-	    if (j < 0) {
-		/* doesn't exist anymore, remove it */
-		m_lineItems[i] &= ~liBPany;
-		updateLineItem(i);
-	    }
-	}
-    }
-
-    // add new breakpoints
-    for (int j = bpt.numBreakpoints()-1; j >= 0; j--) {
-	const Breakpoint& bp = bpt.breakpoint(j);
-	if (fileNamesMatch(bp.fileName, fileName())) {
-	    TRACE(QString().sprintf("updating %s:%d", bp.fileName.data(), bp.lineNo));
-	    int i = bp.lineNo;
-	    if (i < 0 || uint(i) >= m_lineItems.size())
-		continue;
-	    // compute new line item flags for breakpoint
-	    uchar flags = bp.enabled ? liBP : liBPdisabled;
-	    if (bp.temporary)
-		flags |= liBPtemporary;
-	    if (!bp.condition.isEmpty() || bp.ignoreCount != 0)
-		flags |= liBPconditional;
-	    // update if changed
-	    if ((m_lineItems[i] & liBPany) != flags) {
-		m_lineItems[i] &= ~liBPany;
-		m_lineItems[i] |= flags;
-		updateLineItem(i);
-	    }
-	}
-    }
-}
-
-void FileWindow::updateLineItem(int row)
-{
-    updateCell(row, 0);
-}
-
-void FileWindow::setPC(bool set, int lineNo, int frameNo)
-{
-    if (lineNo < 0 || lineNo >= int(m_lineItems.size())) {
-	return;
-    }
-    uchar flag = frameNo == 0  ?  liPC  :  liPCup;
-    if (set) {
-	// set only if not already set
-	if ((m_lineItems[lineNo] & flag) == 0) {
-	    m_lineItems[lineNo] |= flag;
-	    updateLineItem(lineNo);
-	}
-    } else {
-	// clear only if not set
-	if ((m_lineItems[lineNo] & flag) != 0) {
-	    m_lineItems[lineNo] &= ~flag;
-	    updateLineItem(lineNo);
-	}
-    }
-}
-
-void FileWindow::find(const char* text, bool caseSensitive, FindDirection dir)
-{
-    ASSERT(dir == 1 || dir == -1);
-    if (m_texts.size() == 0 || text == 0 || *text == '\0')
-	return;
-
-    int line, dummyCol;
-    cursorPosition(&line, &dummyCol);
-    if (line < 0)
-	line = 0;
-    int curLine = line;			/* remember where we started */
-#if QT_VERSION < 200
-    QString str;
-#else
-    QCString str;
-#endif
-    bool found = false;
-    do {
-	// advance and wrap around
-	line += dir;
-	if (line < 0)
-	    line = m_texts.size()-1;
-	else if (line >= int(m_texts.size()))
-	    line = 0;
-	// search text
-	if (caseSensitive) {
-	    found = strstr(m_texts[line], text) != 0;
-	} else {
-	    int len = strlen(m_texts[line]);
-	    str.setRawData(m_texts[line], len);
-	    found = str.find(text, 0, false) >= 0;
-	    str.resetRawData(m_texts[line], len);
-	}
-    } while (!found && line != curLine);
-
-    scrollTo(line);
-}
-
-void FileWindow::mousePressEvent(QMouseEvent* ev)
-{
-    // Check if right button was clicked.
-    if (ev->button() == RightButton)
-    {
-	emit clickedRight(ev->pos());
-	return;
-    }
-
-    // check if event is in line item column
-    if (findCol(ev->x()) != 0) {
-	KTextView::mousePressEvent(ev);
-	return;
-    }
-
-    // get row
-    int row = findRow(ev->y());
-    if (row < 0)
-	return;
-
-    switch (ev->button()) {
-    case LeftButton:
-	TRACE(QString().sprintf("left-clicked row %d", row));
-	emit clickedLeft(m_fileName, row);
-	break;
-    case MidButton:
-	TRACE(QString().sprintf("mid-clicked row %d", row));
-	emit clickedMid(m_fileName, row);
-	break;
-    default:;
-    }
-}
-
-
 
 WinStack::WinStack(QWidget* parent, const char* name) :
 	QWidget(parent, name),
 	m_activeWindow(0),
 	m_windowMenu(0),
 	m_itemMore(0),
-	m_pcLine(-1)
+	m_pcLine(-1),
+	m_valueTip(this),
+	m_tipLocation(1,1,10,10),
+	m_tabWidth(0)
 {
     // Call menu implementation helper
     initMenu();
@@ -379,6 +40,8 @@ WinStack::WinStack(QWidget* parent, const char* name) :
     connect(&m_findDlg.m_buttonBackward,
 	    SIGNAL(clicked()), SLOT(slotFindBackward()));
 
+    connect(this, SIGNAL(setTabWidth(int)), this, SLOT(slotSetTabWidth(int)));
+
     // Check for right click event.
     connect(this, SIGNAL(clickedRight(const QPoint &)),
 	    SLOT(slotWidgetRightClick(const QPoint &)));
@@ -426,7 +89,7 @@ void WinStack::setWindowMenu(QPopupMenu* menu)
 
 void WinStack::menuCallback(int item)
 {
-    TRACE(QString().sprintf("menu item=0x%x", item));
+    TRACE("menu item=" + QString().setNum(item));
     // check for window
     if ((item & ~ID_WINDOW_INDEX_MASK) == ID_WINDOW_MORE) {
 	selectWindow(item & ID_WINDOW_INDEX_MASK);
@@ -434,9 +97,6 @@ void WinStack::menuCallback(int item)
     }
     
     switch (item) {
-    case ID_FILE_OPEN:
-	openFile();
-	break;
     case ID_FILE_RELOAD:
 	if (m_activeWindow != 0) {
 	    TRACE("reloading one file");
@@ -449,23 +109,28 @@ void WinStack::menuCallback(int item)
 	} else {
 	    m_findDlg.show();
 	}
+    case ID_BRKPT_SET:
+    case ID_BRKPT_TEMP:
+	{
+	    QString file;
+	    int lineNo;
+	    DbgAddr address;
+	    if (activeLine(file, lineNo, address))
+		emit toggleBreak(file, lineNo, address, item == ID_BRKPT_TEMP);
+	}
+	break;
+    case ID_BRKPT_ENABLE:
+	{
+	    QString file;
+	    int lineNo;
+	    DbgAddr address;
+	    if (activeLine(file, lineNo, address))
+		emit enadisBreak(file, lineNo, address);
+	}
+	break;
     }
 }
 
-void WinStack::openFile()
-{
-    QString fileName = KFileDialog::getOpenFileName(m_lastOpenDir, QString(), this);
-    TRACE("openFile: " + fileName);
-    if (fileName.isEmpty()) {
-	return;
-    }
-
-    QFileInfo fi(fileName);
-    m_lastOpenDir = fi.dirPath();
-
-    activateFI(fi, 0);
-}
-
 void WinStack::mousePressEvent(QMouseEvent* mouseEvent)
 {
     // Check if right button was clicked.
@@ -480,20 +145,16 @@ void WinStack::mousePressEvent(QMouseEvent* mouseEvent)
 
 void WinStack::reloadAllFiles()
 {
-    FileWindow* fw;
+    SourceWindow* fw;
     for (fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
 	fw->reloadFile();
     }
 }
 
-void WinStack::activate(const QString& fileName, int lineNo)
+void WinStack::activate(const QString& fileName, int lineNo, const DbgAddr& address)
 {
     QFileInfo fi(fileName);
-    activateFI(fi, lineNo);
-}
 
-bool WinStack::activateFI(QFileInfo& fi, int lineNo)
-{
     if (!fi.isFile()) {
 	/*
 	 * We didn't find that file. Now check if it is a relative path and
@@ -501,21 +162,26 @@ bool WinStack::activateFI(QFileInfo& fi, int lineNo)
 	 */
 	TRACE(fi.filePath() + (" not found, looking in " + m_lastOpenDir));
 	if (!fi.isRelative() || m_lastOpenDir.isEmpty()) {
-	    return false;
+	    return;
 	}
 	fi.setFile(m_lastOpenDir + "/" + fi.filePath());
 	if (!fi.isFile()) {
-	    return false;
+	    return;
 	}
     }
     // if this is not an absolute path name, make it one
-    return activatePath(fi.absFilePath(), lineNo);
+    activatePath(fi.absFilePath(), lineNo, address);
 }
 
-bool WinStack::activatePath(QString pathName, int lineNo)
+void WinStack::activateFile(const QString& fileName)
+{
+    activatePath(fileName, 0, DbgAddr());
+}
+
+bool WinStack::activatePath(QString pathName, int lineNo, const DbgAddr& address)
 {
     // check whether the file is already open
-    FileWindow* fw;
+    SourceWindow* fw;
     for (fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
 	if (fw->fileName() == pathName) {
 	    break;
@@ -523,18 +189,28 @@ bool WinStack::activatePath(QString pathName, int lineNo)
     }
     if (fw == 0) {
 	// not found, load it
-	fw = new FileWindow(pathName, this, "fileWindow");
+	fw = new SourceWindow(pathName, this, "fileWindow");
 	m_fileList.insert(0, fw);
 	connect(fw, SIGNAL(lineChanged()),SIGNAL(lineChanged()));
-	connect(fw, SIGNAL(clickedLeft(const QString&, int)),
-		SIGNAL(toggleBreak(const QString&,int)));
-	connect(fw, SIGNAL(clickedMid(const QString&, int)),
-		SIGNAL(enadisBreak(const QString&,int)));
+	connect(fw, SIGNAL(clickedLeft(const QString&,int,const DbgAddr&,bool)),
+		SIGNAL(toggleBreak(const QString&,int,const DbgAddr&,bool)));
+	connect(fw, SIGNAL(clickedMid(const QString&,int,const DbgAddr&)),
+		SIGNAL(enadisBreak(const QString&,int,const DbgAddr&)));
 
 	// Comunication when right button is clicked.
 	connect(fw, SIGNAL(clickedRight(const QPoint &)),
 		SLOT(slotFileWindowRightClick(const QPoint &)));
 
+	// disassemble code
+	connect(fw, SIGNAL(disassemble(const QString&, int)),
+		SIGNAL(disassemble(const QString&, int)));
+	connect(fw, SIGNAL(expanded(int)), SLOT(slotExpandCollapse(int)));
+	connect(fw, SIGNAL(collapsed(int)), SLOT(slotExpandCollapse(int)));
+
+	// tab width
+	connect(this, SIGNAL(setTabWidth(int)), fw, SLOT(setTabWidth(int)));
+	fw->setTabWidth(m_tabWidth);
+
 	changeWindowMenu();
 	
 	// slurp the file in
@@ -543,13 +219,13 @@ bool WinStack::activatePath(QString pathName, int lineNo)
 	// set PC if there is one
 	emit newFileLoaded();
 	if (m_pcLine >= 0) {
-	    setPC(true, m_pcFile, m_pcLine, m_pcFrame);
+	    setPC(true, m_pcFile, m_pcLine, DbgAddr(m_pcAddress), m_pcFrame);
 	}
     }
-    return activateWindow(fw, lineNo);
+    return activateWindow(fw, lineNo, address);
 }
 
-bool WinStack::activateWindow(FileWindow* fw, int lineNo)
+bool WinStack::activateWindow(SourceWindow* fw, int lineNo, const DbgAddr& address)
 {
     int index = m_fileList.findRef(fw);
     ASSERT(index >= 0);
@@ -568,7 +244,7 @@ bool WinStack::activateWindow(FileWindow* fw, int lineNo)
 
     // make the line visible
     if (lineNo >= 0) {
-	fw->scrollTo(lineNo);
+	fw->scrollTo(lineNo, address);
     }
 
     // first resize the window, then lift it to the top
@@ -578,7 +254,11 @@ bool WinStack::activateWindow(FileWindow* fw, int lineNo)
 
     // set the focus to the new active window
     QWidget* oldActive = m_activeWindow;
+#if QT_VERSION < 200
     fw->setFocusPolicy(QWidget::StrongFocus);
+#else
+    fw->setFocusPolicy(QWidget::WheelFocus);
+#endif
     m_activeWindow = fw;
     if (oldActive != 0 && oldActive != fw) {
 	// disable focus on non-active windows
@@ -592,14 +272,19 @@ bool WinStack::activateWindow(FileWindow* fw, int lineNo)
 }
 
 bool WinStack::activeLine(QString& fileName, int& lineNo)
+{
+    DbgAddr dummy;
+    return activeLine(fileName, lineNo, dummy);
+}
+
+bool WinStack::activeLine(QString& fileName, int& lineNo, DbgAddr& address)
 {
     if (m_activeWindow == 0) {
 	return false;
     }
     
     fileName = m_activeWindow->fileName();
-    int dummyCol;
-    m_activeWindow->cursorPosition(&lineNo, &dummyCol);
+    m_activeWindow->activeLine(lineNo, address);
     return true;
 }
 
@@ -617,7 +302,7 @@ void WinStack::changeWindowMenu()
     // insert current windows
     QString text;
     int index = 1;
-    FileWindow* fw = 0;
+    SourceWindow* fw = 0;
     for (fw = m_fileList.first(); fw != 0 && index < 10; fw = m_fileList.next()) {
 	text.sprintf("&%d ", index);
 	text += fw->fileName();
@@ -630,56 +315,38 @@ void WinStack::changeWindowMenu()
     }
 }
 
-void WinStack::updateLineItems(const BreakpointTable& bpt)
+void WinStack::updateLineItems(const KDebugger* dbg)
 {
-    FileWindow* fw = 0;
+    SourceWindow* fw = 0;
     for (fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
-	fw->updateLineItems(bpt);
+	fw->updateLineItems(dbg);
     }
 }
 
-void WinStack::updatePC(const QString& fileName, int lineNo, int frameNo)
+void WinStack::updatePC(const QString& fileName, int lineNo, const DbgAddr& address, int frameNo)
 {
     if (m_pcLine >= 0) {
-	setPC(false, m_pcFile, m_pcLine, m_pcFrame);
+	setPC(false, m_pcFile, m_pcLine, DbgAddr(m_pcAddress), m_pcFrame);
     }
     m_pcFile = fileName;
     m_pcLine = lineNo;
+    m_pcAddress = address.asString();
     m_pcFrame = frameNo;
     if (lineNo >= 0) {
-	setPC(true, fileName, lineNo, frameNo);
+	setPC(true, fileName, lineNo, address, frameNo);
     }
 }
 
-/*
- * Two file names (possibly full paths) match if the last parts - the file
- * names - match.
- */
-static bool fileNamesMatch(const QString& f1, const QString& f2)
-{
-    // check for null file names first
-    if (f1.isNull() || f2.isNull()) {
-	return f1.isNull() && f2.isNull();
-    }
-
-    /*
-     * Get file names.  Note: Either there is a slash, then skip it, or
-     * there is no slash, then -1 + 1 = 0!
-     */
-    int s1 = f1.findRev('/') + 1;
-    int s2 = f2.findRev('/') + 1;
-    return strcmp(f1.data() + s1, f2.data() + s2) == 0;
-}
-
-void WinStack::setPC(bool set, const QString& fileName, int lineNo, int frameNo)
+void WinStack::setPC(bool set, const QString& fileName, int lineNo,
+		     const DbgAddr& address, int frameNo)
 {
     TRACE((set ? "set PC: " : "clear PC: ") + fileName +
-	  QString().sprintf(":%d#%d", lineNo, frameNo));
+	  QString().sprintf(":%d#%d ", lineNo, frameNo) + address.asString());
     // find file
-    FileWindow* fw = 0;
+    SourceWindow* fw = 0;
     for (fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
-	if (fileNamesMatch(fw->fileName(), fileName)) {
-	    fw->setPC(set, lineNo, frameNo);
+	if (fw->fileNameMatches(fileName)) {
+	    fw->setPC(set, lineNo, address, frameNo);
 	    break;
 	}
     }
@@ -697,14 +364,14 @@ void WinStack::slotFindForward()
 {
     if (m_activeWindow != 0)
 	m_activeWindow->find(m_findDlg.searchText(), m_findDlg.caseSensitive(),
-			     FileWindow::findForward);
+			     SourceWindow::findForward);
 }
 
 void WinStack::slotFindBackward()
 {
     if (m_activeWindow != 0)
 	m_activeWindow->find(m_findDlg.searchText(), m_findDlg.caseSensitive(),
-			     FileWindow::findBackward);
+			     SourceWindow::findBackward);
 }
 
 void WinStack::slotFileWindowRightClick(const QPoint & pos)
@@ -731,6 +398,81 @@ void WinStack::slotWidgetRightClick(const QPoint & pos)
     }
 }
 
+void WinStack::maybeTip(const QPoint& p)
+{
+    if (m_activeWindow == 0)
+	return;
+
+    // get the word at the point
+    QString word;
+    QRect r;
+    if (!m_activeWindow->wordAtPoint(p, word, r))
+	return;
+
+    // must be valid
+    assert(!word.isEmpty());
+    assert(r.isValid());
+
+    // remember the location
+    m_tipLocation = r;
+
+    emit initiateValuePopup(word);
+}
+
+void WinStack::slotShowValueTip(const QString& tipText)
+{
+    m_valueTip.tip(m_tipLocation, tipText);
+}
+
+void WinStack::slotDisassembled(const QString& fileName, int lineNo,
+				const QList<DisassembledCode>& disass)
+{
+    // lookup the file
+    SourceWindow* fw;
+    for (fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
+	if (fw->fileNameMatches(fileName)) {
+	    break;
+	}
+    }
+    if (fw == 0) {
+	// not found: ignore
+	return;
+    }
+
+    fw->disassembled(lineNo, disass);
+}
+
+void WinStack::slotExpandCollapse(int)
+{
+    // update line items after expanding or collapsing disassembled code
+
+    // HACK: we know that this will result in updateLineItems
+    // should be done more cleanly with a separate signal
+    emit newFileLoaded();
+
+    if (m_pcLine >= 0) {
+	setPC(true, m_pcFile, m_pcLine, DbgAddr(m_pcAddress), m_pcFrame);
+    }
+}
+
+
+void WinStack::slotSetTabWidth(int numChars)
+{
+    m_tabWidth = numChars;
+}
+
+
+ValueTip::ValueTip(WinStack* parent) :
+	QToolTip(parent)
+{
+}
+
+void ValueTip::maybeTip(const QPoint& p)
+{
+    WinStack* w = static_cast<WinStack*>(parentWidget());
+    w->maybeTip(p);
+}
+
 
 class MoreWindowsDialog : public QDialog
 {
@@ -800,7 +542,7 @@ void WinStack::selectWindow(int id)
 	// more windows selected: show windows in a list
 	MoreWindowsDialog dlg(this);
 	int i = 0;
-	for (FileWindow* fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
+	for (SourceWindow* fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
 	    dlg.insertString(fw->fileName());
 	    if (m_activeWindow == fw) {
 		index = i;
@@ -815,13 +557,13 @@ void WinStack::selectWindow(int id)
 	index = (id & ID_WINDOW_INDEX_MASK)-1;
     }
 
-    FileWindow* fw = m_fileList.first();
+    SourceWindow* fw = m_fileList.first();
     for (; index > 0; index--) {
 	fw = m_fileList.next();
     }
     ASSERT(fw != 0);
 	
-    activateWindow(fw, -1);
+    activateWindow(fw, -1, DbgAddr());
 }
 
 
@@ -895,3 +637,5 @@ void FindDialog::done(int result)
     QDialog::done(result);
     emit closed();
 }
+
+#include "winstack.moc"
